/*-----------------------------------------------------------------*/
/*! 
  \file calcephspice.c 
  \brief perform general operations on SPICE KERNEL ephemeris data file 
         interpolate SPICE KERNEL Ephemeris data.
         compute position and velocity from SPICE KERNEL ephemeris file.

  \author  M. Gastineau 
           Astronomie et Systemes Dynamiques, IMCCE, CNRS, Observatoire de Paris. 

   Copyright, 2011, 2012, CNRS
   email of the author : gastineau@imcce.fr

  History:                                                                
*/
/*-----------------------------------------------------------------*/

/*-----------------------------------------------------------------*/
/* License  of this file :
 This file is "triple-licensed", you have to choose one  of the three licenses 
 below to apply on this file.
 
    CeCILL-C
    	The CeCILL-C license is close to the GNU LGPL.
    	( http://www.cecill.info/licences/Licence_CeCILL-C_V1-en.html )
   
 or CeCILL-B
        The CeCILL-B license is close to the BSD.
        (http://www.cecill.info/licences/Licence_CeCILL-B_V1-en.txt)
  
 or CeCILL v2.0
      The CeCILL license is compatible with the GNU GPL.
      ( http://www.cecill.info/licences/Licence_CeCILL_V2-en.html )
 

This library is governed by the CeCILL-C, CeCILL-B or the CeCILL license under 
French law and abiding by the rules of distribution of free software.  
You can  use, modify and/ or redistribute the software under the terms 
of the CeCILL-C,CeCILL-B or CeCILL license as circulated by CEA, CNRS and INRIA  
at the following URL "http://www.cecill.info". 

As a counterpart to the access to the source code and  rights to copy,
modify and redistribute granted by the license, users are provided only
with a limited warranty  and the software's author,  the holder of the
economic rights,  and the successive licensors  have only  limited
liability. 

In this respect, the user's attention is drawn to the risks associated
with loading,  using,  modifying and/or developing or reproducing the
software by the user in light of its specific status of free software,
that may mean  that it is complicated to manipulate,  and  that  also
therefore means  that it is reserved for developers  and  experienced
professionals having in-depth computer knowledge. Users are therefore
encouraged to load and test the software's suitability as regards their
requirements in conditions enabling the security of their systems and/or 
data to be ensured and,  more generally, to use and operate it in the 
same conditions as regards security. 

The fact that you are presently reading this means that you have had
knowledge of the CeCILL-C,CeCILL-B or CeCILL license and that you accept its terms.
*/
/*-----------------------------------------------------------------*/

#include "calcephconfig.h"
#if HAVE_STDIO_H
#include <stdio.h>
#endif
#if HAVE_STDLIB_H
#include <stdlib.h>
#endif
#if HAVE_STRING_H
#include <string.h>
#endif
#if HAVE_ERRNO_H
#include <errno.h>
#endif
#if HAVE_MATH_H
#include <math.h>
#endif
#if HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#include "calcephdebug.h"
#include "real.h"
#define __CALCEPH_WITHIN_CALCEPH 1
#include "calceph.h"
#include "calcephinternal.h"
#include "util.h"

static void calceph_spicekernel_close(struct SPICEkernel *eph);
static int calceph_spicekernel_interpol_PV(struct calcephbin_spice *pephbin, struct SPICEkernel *kernel, treal TimeJD0,
                                           treal Timediff, int Target, int Center, stateType * Planet);
static double calceph_spicekernel_getEMRAT(struct SPICEkernel *p_pbinfile);
static int calceph_spicekernel_getconstant(struct SPICEkernel *eph, const char *name, double *p_pdval);
static int calceph_spicekernel_getconstantcount(struct SPICEkernel *eph);
static int calceph_spicekernel_getconstantindex(struct SPICEkernel *eph, int *index,
                                                char name[CALCEPH_MAX_CONSTANTNAME], double *value);
static int calceph_spice_unit_convert(struct calcephbin_spice *pephbin, stateType * Planet, int Target, int InputUnit,
                                      int OutputUnit);
static int calceph_spice_findlibration(struct calcephbin_spice *eph);
static struct TXTFKframe *calceph_spicekernel_findframe(struct SPICEkernel *eph, struct TXTPCKconstant *cst);
static struct TXTFKframe *calceph_spice_findframe(struct calcephbin_spice *eph, struct TXTPCKconstant *cst);

/*--------------------------------------------------------------------------*/
/*! add a file to the list of kernel.
    allocate memory for it
    
  @return  pointer to the new slot 
  
  @param eph (inout) descriptor of the ephemeris.  
*/
/*--------------------------------------------------------------------------*/
struct SPICEkernel *calceph_spice_addfile(struct calcephbin_spice *eph)
{
    struct SPICEkernel *prec = NULL, *pnew;

    if (eph->list == NULL)
    {
        eph->AU = 0.E0;
        eph->EMRAT = 0.E0;
    }
    /* go to the tail of the list of file */
    prec = eph->list;
    if (prec != NULL)
    {
        while (prec->next != NULL)
            prec = prec->next;
    }
    /* allocate memory */
    pnew = (struct SPICEkernel *) malloc(sizeof(struct SPICEkernel));
    if (pnew == NULL)
    {
        fatalerror("Can't allocate memory for SPICEkernel\nSystem error : '%s'\n", strerror(errno));
        return NULL;
    }
    /* add it to the tail */
    pnew->next = NULL;
    if (prec == NULL)
    {
        eph->list = pnew;
    }
    else
    {
        prec->next = pnew;
    }
    return pnew;
}

/*--------------------------------------------------------------------------*/
/*! Close the SPICE ephemeris file.
     
  @return  1 on sucess and 0 on failure
  
  @param eph (inout) descriptor of the ephemeris.  
*/
/*--------------------------------------------------------------------------*/
static void calceph_spicekernel_close(struct SPICEkernel *eph)
{
    switch (eph->filetype)
    {
        case DAF_SPK:
            calceph_spk_close(&eph->filedata.spk);
            break;

        case TXT_PCK:
            calceph_txtpck_close(&eph->filedata.txtpck);
            break;

        case DAF_PCK:
            calceph_binpck_close(&eph->filedata.spk);
            break;

        case TXT_FK:
            calceph_txtfk_close(&eph->filedata.txtfk);
            break;

        default:
            fatalerror("Unknown SPICE type in %d\n", (int) eph->filetype);
            break;
    }
}

/*--------------------------------------------------------------------------*/
/*! Close the SPICE ephemeris file.
     
  @return  1 on sucess and 0 on failure
  
  @param eph (inout) descriptor of the ephemeris.  
*/
/*--------------------------------------------------------------------------*/
void calceph_spice_close(struct calcephbin_spice *eph)
{
    struct SPICEkernel *list = eph->list;

    while (list != NULL)
    {
        struct SPICEkernel *clist = list;

        calceph_spicekernel_close(list);
        list = list->next;
        free(clist);
    }
}

/*--------------------------------------------------------------------------*/
/*!                                                      
     This function read the data block corresponding to the specified Time
     if this block isn't already in memory.
     
     This function works only on all targets.
     for asteroids, the target value is "asteroid number"+CALCEPH_ASTEROID
     
     It computes position and velocity vectors for a selected  
     planetary body from Chebyshev coefficients.          

     return 1 on success
     return 0 on error  
     return -1 if the segment and object is not found
                                                                         
  @param eph (inout) ephemeris descriptor                            
  @param kernel (inout) current kernel                           
  @param TimeJD0+timediff  (in) Time for which position is desired (Julian Date)
  @param Target   (in) Solar system body for which position is desired.         
  @param Center   (in) Solar system body from which position is desired.         
  @param Planet (out) position and veloticity                                                                        
         Planet has position in ua                                                
         Planet has velocity in ua/day
*/
/*--------------------------------------------------------------------------*/
static int calceph_spicekernel_interpol_PV(struct calcephbin_spice *eph, struct SPICEkernel *kernel,
                                           treal TimeJD0, treal Timediff, int Target, int Center, stateType * Planet)
{
    int res = -1;

    Target++;
    Center++;

    switch (kernel->filetype)
    {
        case DAF_SPK:
            res = calceph_spk_interpol_PV(eph, &kernel->filedata.spk, TimeJD0, Timediff, Target, Center, Planet);
            break;

        case TXT_PCK:
            break;

        case DAF_PCK:
            res = calceph_binpck_interpol_PV(eph, &kernel->filedata.spk, TimeJD0, Timediff, Target, Center, Planet);
            break;

        case TXT_FK:
            break;

        default:
            fatalerror("Unknown SPICE type in %d\n", (int) kernel->filetype);
            res = 0;
            break;
    }
#if DEBUG
    printf("calceph_spicekernel_interpol_PV(eph type: %d) returns %d\n", (int) kernel->filetype, res);
#endif
    return res;
}

/*--------------------------------------------------------------------------*/
/*!                                                      
     This function read the data block corresponding to the specified Time
     if this block isn't already in memory.
     
     This function works only on all targets.
     for asteroids, the target value is "asteroid number"+CALCEPH_ASTEROID
     
     It computes position and velocity vectors for a selected  
     planetary body from Chebyshev coefficients.          

                                                                          
  @param eph (inout) ephemeris descriptor                            
  @param TimeJD0+timediff  (in) Time for which position is desired (Julian Date)
  @param Target   (in) Solar system body for which position is desired.         
  @param Center   (in) Solar system body from which position is desired.         
  @param Unit  (in) output unit of the statetype
  @param Planet (out) position and veloticity                                                                        
         Planet has position in ua                                                
         Planet has velocity in ua/day
*/
/*--------------------------------------------------------------------------*/
int calceph_spice_interpol_PV(struct calcephbin_spice *eph, treal TimeJD0, treal Timediff,
                              int Target, int Center, int Unit, stateType * Planet)
{
    int res = -1;
    int ephunit;

    struct SPICEkernel *list = eph->list;

    while (list != NULL && res == -1)
    {
        res = calceph_spicekernel_interpol_PV(eph, list, TimeJD0, Timediff, Target, Center, Planet);
        list = list->next;
    }
    /* check if not found */
    if (res == -1)
    {
        fatalerror("time %.2f isn't in the SPICE file\n", TimeJD0 + Timediff);
    }
    if (res == -1)
        res = 0;
    ephunit = CALCEPH_UNIT_KM + CALCEPH_UNIT_SEC + CALCEPH_UNIT_RAD;
    if (res)
        res = calceph_spice_unit_convert(eph, Planet, Target, ephunit, Unit);
    return res;
}

/*--------------------------------------------------------------------------*/
/*! convert an internal id from calceph to SPICE id 
   e.g., calceph_spice_convertid(eph, 10) return 301
    return -1 if not supported.
  @param eph (inout) ephemerids descriptor                         
   @param target (in) id to be converted
*/
/*--------------------------------------------------------------------------*/
int calceph_spice_convertid(struct calcephbin_spice *eph, int target)
{
    int res;

    switch (target)
    {
        case MOON + 1:
            res = 301;
            break;              /* moon */
        case SS_BARY + 1:
            res = 0;
            break;
        case SUN + 1:
            res = 10;
            break;
        case NUTATIONS + 1:
            res = -1;
            break;
        case LIBRATIONS + 1:   /*res = 301; */
            res = calceph_spice_findlibration(eph);
            break;
        case TTMTDB + 1:
            res = -1;
            break;
        case TCGMTCB + 1:
            res = -1;
            break;
        case (-EARTH + 1):
            res = 399;
            break;              /* earth */

        default:
            res = target;
            break;
    }
    return res;
}

/*--------------------------------------------------------------------------*/
/*! return EMRAT from file 
  @param p_pbinfile (in) ephemeris file
*/
/*--------------------------------------------------------------------------*/
static double calceph_spicekernel_getEMRAT(struct SPICEkernel *p_pbinfile)
{
    double EMRAT = 0.E0, GMEARTH, GMEMB, GMMOON;

    if (calceph_spicekernel_getconstant(p_pbinfile, "BODY301_GM", &GMMOON))
    {
        if (calceph_spicekernel_getconstant(p_pbinfile, "BODY399_GM", &GMEARTH))
        {
            EMRAT = GMEARTH / GMMOON;
        }
        else if (calceph_spicekernel_getconstant(p_pbinfile, "BODY3_GM", &GMEMB))
        {
            EMRAT = GMEMB / GMMOON - 1.E0;
        }
    }
    return EMRAT;
}

/*--------------------------------------------------------------------------*/
/*! return EMRAT from file 
  @param p_pbinfile (in) ephemeris file
*/
/*--------------------------------------------------------------------------*/
double calceph_spice_getEMRAT(struct calcephbin_spice *p_pbinfile)
{
    if (p_pbinfile->EMRAT == 0.E0)
    {
        /* first call : try to find EMRAT */
        struct SPICEkernel *list = p_pbinfile->list;

        while (list != NULL && p_pbinfile->EMRAT == 0.E0)
        {
            p_pbinfile->EMRAT = calceph_spicekernel_getEMRAT(list);
            list = list->next;
        }
    }
    return p_pbinfile->EMRAT;
}

/*--------------------------------------------------------------------------*/
/*! return AU from file 
  @param p_pbinfile (in) ephemeris file
*/
/*--------------------------------------------------------------------------*/
double calceph_spice_getAU(struct calcephbin_spice *p_pbinfile)
{
    if (p_pbinfile->AU == 0.E0)
    {
        calceph_spice_getconstant(p_pbinfile, "AU", &p_pbinfile->AU);
    }
    return p_pbinfile->AU;
}

/*--------------------------------------------------------------------------*/
/*! return the value of the specified constant from file 
  return 0 if constant is not found
  return 1 if constant is found and set p_pdval.
  @param eph (in) ephemeris file
  @param name (in) constant name
  @param p_pdval (out) value of the constant
*/
/*--------------------------------------------------------------------------*/
static int calceph_spicekernel_getconstant(struct SPICEkernel *eph, const char *name, double *p_pdval)
{
    int found = 0;

    switch (eph->filetype)
    {
        case TXT_PCK:
            found = calceph_txtpck_getconstant(&eph->filedata.txtpck, name, p_pdval);
            break;

        case TXT_FK:
            found = calceph_txtpck_getconstant(&eph->filedata.txtfk.txtpckfile, name, p_pdval);
            break;

        default:
            break;
    }
    return found;
}

/*--------------------------------------------------------------------------*/
/*! return the value of the specified constant from file 
  return 0 if constant is not found
  return 1 if constant is found and set p_pdval.
  @param eph (in) ephemeris file
  @param name (in) constant name
  @param p_pdval (out) value of the constant
*/
/*--------------------------------------------------------------------------*/
int calceph_spice_getconstant(struct calcephbin_spice *eph, const char *name, double *p_pdval)
{
    int found = 0;
    struct SPICEkernel *list = eph->list;

    while (list != NULL && found == 0)
    {
        found = calceph_spicekernel_getconstant(list, name, p_pdval);
        list = list->next;
    }
    return found;
}

/*--------------------------------------------------------------------------*/
/*! return the specified constant from file 
  return NULL if constant is not found
  @param eph (in) ephemeris file
  @param name (in) constant name
*/
/*--------------------------------------------------------------------------*/
static struct TXTPCKconstant *calceph_spicekernel_getptrconstant(struct SPICEkernel *eph, const char *name)
{
    struct TXTPCKconstant *found = NULL;

    switch (eph->filetype)
    {
        case TXT_PCK:
            found = calceph_txtpck_getptrconstant(&eph->filedata.txtpck, name);
            break;

        case TXT_FK:
            found = calceph_txtpck_getptrconstant(&eph->filedata.txtfk.txtpckfile, name);
            break;

        default:
            break;
    }
    return found;
}

/*--------------------------------------------------------------------------*/
/*! return the specified constant from file 
   return NULL if constant is not found
   @param eph (in) ephemeris file
   @param name (in) constant name
*/
/*--------------------------------------------------------------------------*/
static struct TXTPCKconstant *calceph_spice_getptrconstant(struct calcephbin_spice *eph, const char *name)
{
    struct TXTPCKconstant *found = NULL;
    struct SPICEkernel *list = eph->list;

    while (list != NULL && found == NULL)
    {
        found = calceph_spicekernel_getptrconstant(list, name);
        list = list->next;
    }
    return found;
}

/*--------------------------------------------------------------------------*/
/*! return the number of constants available in the ephemeris file 
  
  @param eph (inout) ephemeris descriptor
*/
/*--------------------------------------------------------------------------*/
static int calceph_spicekernel_getconstantcount(struct SPICEkernel *eph)
{
    int res = 0;

    switch (eph->filetype)
    {
        case TXT_PCK:
            res = calceph_txtpck_getconstantcount(&eph->filedata.txtpck);
            break;

        case TXT_FK:
            res = calceph_txtpck_getconstantcount(&eph->filedata.txtfk.txtpckfile);
            break;

        default:
            break;
    }
    return res;
}

/*--------------------------------------------------------------------------*/
/*! return the number of constants available in the ephemeris file 
  
  @param eph (inout) ephemeris descriptor
*/
/*--------------------------------------------------------------------------*/
int calceph_spice_getconstantcount(struct calcephbin_spice *eph)
{
    int res = 0;
    struct SPICEkernel *list = eph->list;

    while (list != NULL)
    {
        res += calceph_spicekernel_getconstantcount(list);
        list = list->next;
    }
    return res;
}

/*--------------------------------------------------------------------------*/
/*! get the name and its value of the constant available 
    at some index from the ephemeris file 
   return 1 if index is valid
   return 0 if index isn't valid
  
  @param eph (inout) ephemeris descriptor
  @param index (inout) index of the constant (must be between 1 and calceph_getconstantcount() )
            if index > number of constant in this kernel, decrement it
  @param name (out) name of the constant 
  @param value (out) value of the constant 
*/
/*--------------------------------------------------------------------------*/
static int calceph_spicekernel_getconstantindex(struct SPICEkernel *eph, int *index,
                                                char name[CALCEPH_MAX_CONSTANTNAME], double *value)
{
    int res = 0;

    switch (eph->filetype)
    {
        case TXT_PCK:
            res = calceph_txtpck_getconstantindex(&eph->filedata.txtpck, index, name, value);
            break;

        case TXT_FK:
            res = calceph_txtpck_getconstantindex(&eph->filedata.txtfk.txtpckfile, index, name, value);
            break;

        default:
            break;
    }
    return res;
}

/*--------------------------------------------------------------------------*/
/*! get the name and its value of the constant available 
    at some index from the ephemeris file 
   return 1 if index is valid
   return 0 if index isn't valid
  
  @param eph (inout) ephemeris descriptor
  @param index (in) index of the constant (must be between 1 and calceph_getconstantcount() )
  @param name (out) name of the constant 
  @param value (out) value of the constant 
*/
/*--------------------------------------------------------------------------*/
int calceph_spice_getconstantindex(struct calcephbin_spice *eph, int index, char name[CALCEPH_MAX_CONSTANTNAME],
                                   double *value)
{
    int res = 0;
    struct SPICEkernel *list = eph->list;

    while (list != NULL && index > 0 && res == 0)
    {
        res = calceph_spicekernel_getconstantindex(list, &index, name, value);
        list = list->next;
    }
    return res;
}

/*--------------------------------------------------------------------------*/
/*!                                                      
     This function changes the state vector, which is expressed in InputUnit units as input,  
       to the OutputUnit as output.
     
     This function is for SPK ephemerids format.

  it returns 0 if the function fails to convert.
                                                                          
  @param pephbin (inout) ephemerids descriptor                         
  @param Target   (in) Solar system body for which position is desired.         
  @param InputUnit  (in) output unit of the statetype
  @param OutputUnit  (in) output unit of the statetype
  @param Planet (inout) position and veloticity                                                                        
          Input : expressed in InputUnit units.
          Output : expressed in OutputUnit units.
*/
/*--------------------------------------------------------------------------*/
static int calceph_spice_unit_convert(struct calcephbin_spice *pephbin, stateType * Planet,
                                      int Target, int InputUnit, int OutputUnit)
{
    int res = 1;
    int i;

    /* if same unit in input and output => do nothing */
    if (InputUnit == OutputUnit)
        return res;
    /* some checks */

    switch (Target)
    {
        case LIBRATIONS:       /*Librations */
            if ((OutputUnit & CALCEPH_UNIT_RAD) == 0)
                fatalerror("Units for libration must be in radians\n");
            if ((InputUnit & CALCEPH_UNIT_RAD) == 0)
                fatalerror("Input units for libration must be in radians\n");
            if (res)
                res = calceph_unit_convert_velocity_time(Planet, InputUnit, OutputUnit);
            break;

        case TCGMTCB:          /*TCG-TCB */
        case TTMTDB:           /*TT-TDB */
            if ((OutputUnit & CALCEPH_UNIT_DAY) == CALCEPH_UNIT_DAY
                && (InputUnit & CALCEPH_UNIT_SEC) == CALCEPH_UNIT_SEC)
            {
                Planet->Position[0] *= 86400E0;
            }
            if ((InputUnit & CALCEPH_UNIT_DAY) == CALCEPH_UNIT_DAY
                && (OutputUnit & CALCEPH_UNIT_SEC) == CALCEPH_UNIT_SEC)
            {
                Planet->Position[0] /= 86400E0;
            }
            if ((OutputUnit & (CALCEPH_UNIT_DAY + CALCEPH_UNIT_SEC)) == 0)
            {
                res = 0;
                fatalerror("Units must include CALCEPH_UNIT_DAY or CALCEPH_UNIT_SEC \n");
            }
            if ((OutputUnit & (CALCEPH_UNIT_DAY + CALCEPH_UNIT_SEC)) == CALCEPH_UNIT_DAY + CALCEPH_UNIT_SEC)
            {
                res = 0;
                fatalerror("Units must include only one value : CALCEPH_UNIT_DAY or CALCEPH_UNIT_SEC \n");
            }

        case NUTATIONS:
            break;

        default:
            if ((OutputUnit & (CALCEPH_UNIT_AU + CALCEPH_UNIT_KM)) == 0)
            {
                res = 0;
                fatalerror("Units must include CALCEPH_UNIT_AU or CALCEPH_UNIT_KM \n");
            }
            if ((OutputUnit & (CALCEPH_UNIT_AU + CALCEPH_UNIT_KM)) == CALCEPH_UNIT_AU + CALCEPH_UNIT_KM)
            {
                res = 0;
                fatalerror("Units must include only one value : CALCEPH_UNIT_AU or CALCEPH_UNIT_KM \n");
            }

            if ((OutputUnit & CALCEPH_UNIT_KM) == CALCEPH_UNIT_KM && (InputUnit & CALCEPH_UNIT_AU) == CALCEPH_UNIT_AU)
            {
                treal rau = calceph_spice_getAU(pephbin);

                if (rau == 0.E0)
                {
                    res = 0;
                    fatalerror("Astronomical unit is not available in the ephemeris file\n");
                }
                for (i = 0; i < 3; i++)
                {
                    Planet->Position[i] *= rau;
                    Planet->Velocity[i] *= rau;
                }
            }
            if ((InputUnit & CALCEPH_UNIT_KM) == CALCEPH_UNIT_KM && (OutputUnit & CALCEPH_UNIT_AU) == CALCEPH_UNIT_AU)
            {
                treal rau = calceph_spice_getAU(pephbin);

                if (rau == 0.E0)
                {
                    res = 0;
                    fatalerror("Astronomical unit is not available in the ephemeris file\n");
                }
                for (i = 0; i < 3; i++)
                {
                    Planet->Position[i] /= rau;
                    Planet->Velocity[i] /= rau;
                }
            }
            if (res)
                res = calceph_unit_convert_velocity_time(Planet, InputUnit, OutputUnit);
            break;
    }
    return res;
}

/*--------------------------------------------------------------------------*/
/*! return the object id of the euler angles of the moon (libration)
    return -1 if not supported.
  @param eph (inout) ephemerids descriptor                         
*/
/*--------------------------------------------------------------------------*/
int calceph_spice_findlibration(struct calcephbin_spice *eph)
{
    struct TXTPCKconstant *pcstOBJECT_MOON_FRAME;
    struct TXTFKframe *frame;

    pcstOBJECT_MOON_FRAME = calceph_spice_getptrconstant(eph, "OBJECT_MOON_FRAME");
    if (pcstOBJECT_MOON_FRAME == NULL)
    {
#if DEBUG
        printf("'OBJECT_MOON_FRAME' not found in the kernels\n");
#endif
        return -1;
    }
    frame = calceph_spice_findframe(eph, pcstOBJECT_MOON_FRAME);
    if (frame == NULL)
    {
#if DEBUG
        printf("frame not found in the kernels\n");
#endif
        return -1;
    }
    if (frame->center != 301)
    {
#if DEBUG
        printf("center is not the moon\n");
#endif
        return -1;
    }
#if DEBUG
    printf("class_id for the libration of the moon : %d\n", frame->class_id);
#endif
    return frame->class_id;
}

/*--------------------------------------------------------------------------*/
/*! return the frame associated to the specified value from this kernel
    return NULL if not found.
  @param eph (inout) kernel
  @param cst (in) constant to find
*/
/*--------------------------------------------------------------------------*/
static struct TXTFKframe *calceph_spicekernel_findframe(struct SPICEkernel *eph, struct TXTPCKconstant *cst)
{
    struct TXTFKframe *frame = NULL;

    switch (eph->filetype)
    {
        case TXT_FK:
            frame = calceph_txtfk_findframe(&eph->filedata.txtfk, cst);
            break;

        default:
            break;
    }
    return frame;
}

/*--------------------------------------------------------------------------*/
/*! return the frame associated to the specified value from a list of kernel
    return NULL if not found.
  @param eph (inout) ephemerids descriptor                         
  @param cst (in) constant to find
*/
/*--------------------------------------------------------------------------*/
static struct TXTFKframe *calceph_spice_findframe(struct calcephbin_spice *eph, struct TXTPCKconstant *cst)
{
    struct TXTFKframe *frame = NULL;
    struct SPICEkernel *list = eph->list;

    while (list != NULL && frame == NULL)
    {
        frame = calceph_spicekernel_findframe(list, cst);
        list = list->next;
    }
    return frame;
}
